@@ -2,7 +2,7 @@ package MVC::Neaf::Request::PSGI;
 
 use strict;
 use warnings;
-our $VERSION = 0.18;
+our $VERSION = 0.19;
 
 =head1 NAME
 
@@ -16,6 +16,17 @@ use URI::Escape qw(uri_unescape);
 use Encode;
 use Plack::Request;
 
+# NOTE HACK - prevent 'Can't locate object method seek via package IO::Handle'
+# try preloading it by hand (errors ignored)
+eval { require FileHandle }
+    if $] < 5.014;
+# NOTE HACK - prevent load-time warnings from Cookie::Baker
+#     which we aren't even using
+eval {
+    $SIG{__WARN__} = sub {};
+    require Cookie::Baker;
+};
+
 use parent qw(MVC::Neaf::Request);
 
 =head2 new( env => $psgi_input )
@@ -28,6 +39,8 @@ my %default_env = (
     REQUEST_METHOD => 'GET',
 );
 
+# TODO 0.30 rewrite env copying for good.
+# Maybe separate ::GET and ::POST to avoid if's
 sub new {
     my $class = shift;
 
